/* ----------------------------------------------------------------
 * BIFROST* ENGINE DEMO - converted to z88dk C Compiler
 *
 * This program can be compiled as follows:
 *
 * zcc.exe +zx -lm -lndos -lbifrost -create-app -O2 bifrostdem.c -obifrostdem.bin
 *
 * After compiling it, use the following loader to execute:
 *
 * 10 CLEAR VAL "32767"
 * 20 LOAD "TILES"CODE
 * 30 LOAD "BIFROST*"CODE
 * 40 LOAD "bifrostdem"CODE
 * 50 RANDOMIZE USR VAL "32768"
 *
 * Original version and further information is available at
 * http://www.worldofspectrum.org/infoseekid.cgi?id=0027405
 * ----------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <input.h>

#include "bifrost.h"

#define printMode(mode)      printf("\x01%c", mode)
#define printCls()           printf("\x0c")
#define printInk(k)          printf("\x10%c", '0'+k)
#define printPaper(k)        printf("\x11%c", '0'+k)
#define printAt(row, col)    printf("\x16%c%c", ' '+row, ' '+2*col)
#define printAt64c(row, col) printf("\x16%c%c", ' '+row, ' '+col)

void pressAnyKey() {
    in_WaitForNoKey();

    printInk(7);
    printAt(22, 1);
    printf("PRESS ANY KEY");

    in_WaitForKey();

    printAt(22, 1);
    printf("             ");
}

void pressSpeedKey() {
    int f, key;

    for (f = 0; f < 51; ++f) {
        BIFROST_setTile(rand()%9, rand()%9, rand()%8);
    }
    in_WaitForNoKey();

    printInk(7);
    printAt(22, 1);
    printf("CHOOSE SPEED 2-4 OR 0 TO EXIT");

    while ((key = in_Inkey()) != '0') {
        switch (key) {
            case '2':
                BIFROST_resetAnimSlow();
                printInk(5);
                printAt(20, 25);
                printf(" slow)");
                break;
            case '4':
                BIFROST_resetAnimFast();
                printInk(5);
                printAt(20, 25);
                printf(" fast)");
                break;
        }
    }

    printAt(22, 1);
    printf("                             ");
}

main()
{
    int f, g, a, b, c;

    randomize();

    printMode(32);
    zx_border(0);
    printPaper(0);
    printCls();

    printInk(6);
    printAt(4, 22);
    printf("BIFROST*");
    printAt(5, 23);
    printf("ENGINE");
    printAt(6, 24);
    printf("DEMO");
    printInk(4);
    printAt(10, 24);
    printf("with");
    printAt(11, 23);
    printf("z88dk!");

    BIFROST_start();
    for (f = 0; f < 81; ++f) {
        BIFROST_tilemap[f] = BIFROST_STATIC + f;
    }

    while (1) {
        pressAnyKey();

        printInk(5);
        printAt(20, 1);
        printf("Demonstrating static tiles");

        for (f = 0; f < 81; ++f) {
            BIFROST_tilemap[f] = BIFROST_STATIC + (rand()%26)+8;
        }
        pressAnyKey();

        printInk(5);
        printAt(20, 1);
        printf("Animated tiles (4 frames) ");

        pressSpeedKey();

        printInk(5);
        printAt(20, 17);
        printf("2");

        BIFROST_resetAnim2Frames();
        pressSpeedKey();
        BIFROST_resetAnim4Frames();

        printInk(5);
        printAt(20, 1);
        printf("Directly modifying areas      ");

        M_BIFROST_SETTILE(4, 4, BIFROST_DISABLED);
        M_BIFROST_SETTILE(4, 5, BIFROST_DISABLED);

        printAt(9, 9);
        printf("BIFR");
        printAt(10, 9);
        printf("OST*");

        for (c = 0; c < 8; ++c) {
            a = rand()%8;
            for (b = 0; b < 2; ++b) {
                for (f = 8+72+0; f < 8+72+16; ++f) {
                    for (g = 9; g < 13; ++g) {
                        *BIFROST_findAttrH(f, g) = (b == 0 ? a*8 : a);
                        a = (a == 7 ? 3 : a+1);
                    }
                }
            }
        }
    }
}

